import { transform } from 'lodash';
import * as _ from 'lodash';
import { HttpStatusCode } from 'azure-functions-ts-essentials';
import { Typegoose } from 'typegoose';

class BaseDocument extends Typegoose {
}

var ErrorType;
(function (ErrorType) {
    ErrorType["Missing"] = "missing";
    ErrorType["MissingField"] = "missing_field";
    ErrorType["Invalid"] = "invalid";
    ErrorType["AlreadyExists"] = "already_exists";
})(ErrorType || (ErrorType = {}));

function connect(instance, connStr) {
    return new Promise((resolve, reject) => instance.connect(connStr, { useMongoClient: true }, (err) => err
        ? reject(err)
        : resolve()));
}
function parseFields(rawFields) {
    if (!rawFields)
        return {};
    return rawFields.split(',')
        .map(cur => String(cur).trim())
        .reduce((acc, cur) => {
        acc[cur] = 1;
        return acc;
    }, {});
}
function parsePopulation(rawPopulation) {
    if (!rawPopulation)
        return '';
    let obj = {};
    for (const item of rawPopulation.split(','))
        obj = appendObject(obj, item);
    return toPopulation(obj);
}
const appendObject = (obj, path) => {
    const keys = path.split(':');
    const lastKey = keys.pop();
    const lastObj = keys.reduce((acc, cur) => acc[cur] = acc[cur] || {}, obj);
    lastObj[lastKey] = 'path';
    return obj;
};
const toPopulation = (obj) => {
    return transform(obj, (res, value, key) => {
        if (typeof (value) === 'object')
            res.push({
                path: key,
                populate: toPopulation(value)
            });
        else
            res.push({ path: key });
    }, []);
};
const getErrorResponse = (err) => {
    let status = HttpStatusCode.InternalServerError;
    let type = '';
    if (err.name === 'MongoError' && err.code === 11000) {
        status = HttpStatusCode.Conflict;
        type = ErrorType.AlreadyExists;
    }
    else if (err.name === 'ValidationError') {
        status = HttpStatusCode.UnprocessableEntity;
        type = ErrorType.MissingField;
    }
    return {
        status,
        body: {
            type,
            message: err.message
        }
    };
};
function clearCollection(instance, name) {
    const collection = instance.connection.collections[name];
    return collection.drop()
        .catch(() => clearCollection(instance, name));
}
class Mongooser {
    constructor(model) {
        this.model = model;
    }
    getOne(id, projection, population) {
        const query = this.model.findOne({ _id: id }, projection).populate(population).lean();
        return query.then((doc) => {
            if (!doc)
                return Promise.resolve({
                    status: HttpStatusCode.NotFound
                });
            const data = doc;
            data._id = String(doc._id);
            return {
                status: HttpStatusCode.OK,
                body: Object.assign({ _id: data._id }, JSON.parse(JSON.stringify(data)))
            };
        })
            .catch(getErrorResponse);
    }
    getMany(projection, showInactive = false, population) {
        const query = this.model.find(!showInactive
            ? { isActive: true }
            : {}, projection).populate(population).lean();
        return query
            .then((docs) => {
            const data = [];
            for (const item of docs) {
                item._id = String(item._id);
                data.push(Object.assign({ _id: item._id }, JSON.parse(JSON.stringify(item))));
            }
            return {
                status: HttpStatusCode.OK,
                body: {
                    data,
                    hasMore: false,
                    totalCount: data.length
                }
            };
        })
            .catch(getErrorResponse);
    }
    insertMany(req) {
        const contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        return this.model.insertMany(req.body)
            .then((docs) => {
            const data = [];
            for (const item of docs) {
                item._id = String(item._id);
                data.push(Object.assign({ _id: item._id }, JSON.parse(JSON.stringify(item))));
            }
            return {
                status: HttpStatusCode.Created,
                body: {
                    data,
                    hasMore: false,
                    totalCount: data.length
                }
            };
        })
            .catch(getErrorResponse);
    }
    updateOne(req, id) {
        const contentType = req.headers ? req.headers['content-type'] : undefined;
        if (!(contentType && contentType.indexOf('application/json') >= 0))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!(req.body && Object.keys(req.body).length))
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        if (!id)
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Invalid
                }
            });
        return this.model.findOneAndUpdate({ _id: id }, req.body, { new: true }).lean()
            .then((doc) => {
            if (!doc)
                return {
                    status: HttpStatusCode.BadRequest,
                    body: {
                        type: ErrorType.Missing
                    }
                };
            else {
                const data = doc;
                data._id = String(doc._id);
                return {
                    status: HttpStatusCode.OK,
                    body: Object.assign({ _id: data._id }, JSON.parse(JSON.stringify(data)))
                };
            }
        })
            .catch(getErrorResponse);
    }
    deactivateOne(id) {
        if (!id)
            return Promise.resolve({
                status: HttpStatusCode.BadRequest,
                body: {
                    type: ErrorType.Missing
                }
            });
        return this.model.findOneAndUpdate({ _id: id }, { isActive: false }).lean()
            .then((doc) => {
            if (!doc)
                return {
                    status: HttpStatusCode.BadRequest
                };
            else
                return {
                    status: HttpStatusCode.OK,
                    body: {
                        deactivated: true,
                        _id: String(doc._id)
                    }
                };
        })
            .catch(getErrorResponse);
    }
}

export { BaseDocument, ErrorType, connect, parseFields, parsePopulation, clearCollection, Mongooser };
//# sourceMappingURL=azure-functions-mongooser.js.map
